@@ -30,8 +30,6 @@
 
 #include <KStandardDirs>
 
-#include <ktexteditor/smartrange.h>
-#include <ktexteditor/smartinterface.h>
 #include <ktexteditor/document.h>
 
 #include <language/duchain/topducontext.h>
@@ -53,7 +51,9 @@ using namespace KTextEditor;
 namespace Python
 {
 
-ContextBuilder::ContextBuilder() : m_indentInformationCache(0)
+ContextBuilder::ContextBuilder()
+    : m_indentInformationCache(0)
+    , m_prebuilding(false)
 {
 
 }
@@ -328,19 +328,30 @@ void ContextBuilder::visitCode(CodeAst* node) {
     AstDefaultVisitor::visitCode(node);
 }
 
-QPair<KUrl, QStringList> ContextBuilder::findModulePath(const QString& name)
+QPair<KUrl, QStringList> ContextBuilder::findModulePath(const QString& name, const KUrl& currentDocument)
 {
     QStringList nameComponents = name.split(".");
     QList<KUrl> searchPaths;
     if ( name.startsWith('.') ) {
-        // This is used for "from .foo import" or similar (relative imports)
+        /* To take care for imports like "from ....xxxx.yyy import zzz"
+         * we need to take current doc path and run "cd .." enough times
+         */
         nameComponents.removeFirst();
-        searchPaths << currentlyParsedDocument().toUrl().directory();
+        QString tname = name.mid(1); // remove first dot
+        QDir curPathDir = QDir(currentDocument.directory());
+        foreach(QString c, tname) {
+            if (c != ".")
+                break;
+            curPathDir.cdUp();
+            nameComponents.removeFirst();
+        }
+        searchPaths << curPathDir.path();
     }
     else {
         // If this is not a relative import, use the project directory,
         // the current directory, and all system include paths.
-        searchPaths = Helper::getSearchPaths(currentlyParsedDocument().toUrl());
+        // FIXME: If absolute imports enabled, don't add curently parsed doc path
+        searchPaths = Helper::getSearchPaths(currentDocument);
     }
     // Loop over all the name components, and find matching folders or files.
     KUrl tmp;
@@ -364,7 +375,6 @@ QPair<KUrl, QStringList> ContextBuilder::findModulePath(const QString& name)
             QFile sourcefile(testFilename + ".py");
             QFileInfo sourcedir(testFilename);
             tmp.cd(component);
-            kDebug() << testFilename << "exists: [file/dir]" << sourcefile.exists() << sourcedir.exists();
             if ( ! sourcedir.exists() || ! sourcedir.isDir() || leftNameComponents.isEmpty() ) {
                 // If the search cannot continue further down into a hierarchy of directories,
                 // the file matching the next name component will be returned,
@@ -447,7 +457,6 @@ void ContextBuilder::visitFunctionArguments(FunctionDefinitionAst* node)
     // The DUChain expects the context containing a function's arguments to be of type Function.
     // The function body will have DUContext::Other as type, as it contains only code.
     DUContext* funcctx = openContext(node->arguments, range, DUContext::Function, node->name);
-    kDebug() << funcctx;
     visitNode(node->arguments);
     closeContext();
     // the parameters should be visible in the function body, so import that context there
