@@ -46,7 +46,7 @@ using namespace KDevelop;
 namespace Python {
 
 QList<KUrl> Helper::cachedSearchPaths;
-QString Helper::dataDir;
+QStringList Helper::dataDirs;
 QString Helper::documentationFile;
 DUChainPointer<TopDUContext> Helper::documentationFileContext = DUChainPointer<TopDUContext>(0);
 
@@ -59,6 +59,22 @@ AbstractType::Ptr Helper::resolveType(AbstractType::Ptr type)
         return type;
 }
 
+Declaration* Helper::accessAttribute(Declaration* accessed, const QString& attribute, DUContext* current)
+{
+    if ( ! accessed || ! accessed->type<StructureType>() ) {
+        return 0;
+    }
+    StructureType::Ptr type = accessed->type<StructureType>();
+    QList<DUContext*> searchContexts = Helper::internalContextsForClass(type, current->topContext());
+    foreach ( DUContext* c, searchContexts ) {
+        QList< Declaration* > found = c->findLocalDeclarations(KDevelop::Identifier(attribute));
+        if ( ! found.isEmpty() ) {
+            return found.first();
+        }
+    }
+    return 0;
+}
+
 AbstractType::Ptr Helper::extractTypeHints(AbstractType::Ptr type, TopDUContext* current)
 {
     if ( type ) {
@@ -178,7 +194,6 @@ Declaration* Helper::declarationForName(NameAst* /*ast*/, const QualifiedIdentif
     QList<Declaration*> declarations;
     QList<Declaration*> localDeclarations;
     QList<Declaration*> importedLocalDeclarations;
-    kDebug() << "Finding declaration for name" << identifier.toString() << " before " << nodeRange.end << ", in context" << context->range();
     {
         DUChainReadLocker lock(DUChain::lock());
         if ( context.data() == context->topContext() and nodeRange.isValid() ) {
@@ -191,10 +206,9 @@ Declaration* Helper::declarationForName(NameAst* /*ast*/, const QualifiedIdentif
                                                            AbstractType::Ptr(0), DUContext::DontResolveAliases);
         importedLocalDeclarations = context->findDeclarations(identifier.last(), nodeRange.end);
     }
-    Declaration* declaration;
+    Declaration* declaration = 0;
     if ( localDeclarations.length() ) {
         declaration = localDeclarations.last();
-        kDebug() << "Using local declaration";
     }
     else if ( importedLocalDeclarations.length() ) {
         // don't use declarations from class decls, they must be referenced through "self.<foo>"
@@ -208,13 +222,11 @@ Declaration* Helper::declarationForName(NameAst* /*ast*/, const QualifiedIdentif
                 break;
             }
         } while ( not importedLocalDeclarations.isEmpty() );
-        kDebug() << "Using imported local declaration (i.e., argument)";
     }
-    else if ( declarations.length() ) {
+
+    if ( !declaration && declarations.length() ) {
         declaration = declarations.last();
-        kDebug() << "Using global declaration";
     }
-    else declaration = 0;
     return declaration;
 }
 
@@ -246,18 +258,19 @@ Declaration* Helper::resolveAliasDeclaration(Declaration* decl)
 {
     AliasDeclaration* alias = dynamic_cast<AliasDeclaration*>(decl);
     if ( alias ) {
+        DUChainReadLocker lock;
         return alias->aliasedDeclaration().data();
     }
     else
         return decl;
 }
 
-QString Helper::getDataDir() {
-    if ( Helper::dataDir.isNull() ) {
+QStringList Helper::getDataDirs() {
+    if ( Helper::dataDirs.isEmpty() ) {
         KStandardDirs d;
-        Helper::dataDir = d.findDirs("data", "kdevpythonsupport/documentation_files").first();
+        Helper::dataDirs = d.findDirs("data", "kdevpythonsupport/documentation_files");
     }
-    return Helper::dataDir;
+    return Helper::dataDirs;
 }
 
 QString Helper::getDocumentationFile() {
@@ -289,7 +302,9 @@ QList<KUrl> Helper::getSearchPaths(KUrl workingOnDocument)
         searchPaths.append(KUrl(project->folder().url()));
     }
     
-    searchPaths.append(KUrl(getDataDir()));
+    foreach ( const QString& path, getDataDirs() ) {
+        searchPaths.append(KUrl(path));
+    }
     
     if ( cachedSearchPaths.isEmpty() ) {
         KStandardDirs d;
