@@ -24,6 +24,7 @@
 #include "pythonduchainexport.h"
 #include "types/unsuretype.h"
 #include "ast.h"
+#include "expressionvisitor.h"
 
 #include <interfaces/iproject.h>
 #include <language/duchain/declaration.h>
@@ -46,11 +47,11 @@ class KDEVPYTHONDUCHAIN_EXPORT Helper {
 public:
     /** get search paths for python files **/
     static QList<KUrl> getSearchPaths(KUrl workingOnDocument);
-    static QString dataDir;
+    static QStringList dataDirs;
     static QString documentationFile;
     static DUChainPointer<TopDUContext> documentationFileContext;
     
-    static QString getDataDir();
+    static QStringList getDataDirs();
     static QString getDocumentationFile();
     static ReferencedTopDUContext getDocumentationFileContext();
     
@@ -59,6 +60,8 @@ public:
     static AbstractType::Ptr extractTypeHints(AbstractType::Ptr type, TopDUContext* current);
     
     static AbstractType::Ptr resolveType(AbstractType::Ptr type);
+
+    static Declaration* accessAttribute(Declaration* accessed, const QString& attribute, DUContext* current);
     
     /**
      * @brief Finds whether the specified called declaration is a function declaration, and, if not, checks for a class declaration; then returns the constructor
@@ -70,8 +73,9 @@ public:
     
     template<typename T> static const Decorator* findDecoratorByName(T* inDeclaration, const QString& name) {
         register int count = inDeclaration->decoratorsSize();
+        const IndexedString indexedName = IndexedString(name);
         for ( int i = 0; i < count; i++ ) {
-            if ( inDeclaration->decorators()[i].name() == name )
+            if ( inDeclaration->decorators()[i].fastName() == indexedName )
                 return &(inDeclaration->decorators()[i]);
         }
         return 0;
