@@ -24,6 +24,8 @@
 
 #include "pyduchaintest.h"
 
+#include <stdlib.h>
+
 #include <language/duchain/topducontext.h>
 #include <language/duchain/dumpchain.h>
 #include <language/codegen/coderepresentation.h>
@@ -60,9 +62,72 @@ using namespace Python;
 
 PyDUChainTest::PyDUChainTest(QObject* parent): QObject(parent), m_pool()
 {
+    assetsDir = QDir(DUCHAIN_PY_DATA_DIR);
+    if (!assetsDir.cd("data")) {
+        qFatal("Failed find data directory for test files. Aborting");
+    }
+
+    char tempdirname[] = "/tmp/kdev-python-test.XXXXXX";
+    if (!mkdtemp(tempdirname)) {
+        qFatal("Failed to create temp directory, Aboring");
+    }
+    testDir = QDir(QString(tempdirname));
+    kDebug() << "tempdirname" << tempdirname;
+
+    QByteArray pythonpath = qgetenv("PYTHONPATH");
+    pythonpath.prepend(":").prepend(assetsDir.absolutePath().toAscii());
+    qputenv("PYTHONPATH", pythonpath);
+
     initShell();
 }
 
+QList<QString> PyDUChainTest::FindPyFiles(QDir& rootDir)
+{
+    QList<QString> foundfiles;
+    rootDir.setFilter(QDir::Files | QDir::Dirs | QDir::NoDot | QDir::NoDotDot);
+    rootDir.setNameFilters(QStringList() << "*.py"); // We only want .py files
+    
+    QDirIterator it(rootDir, QDirIterator::Subdirectories);
+    while(it.hasNext()) {
+        foundfiles.append(it.next());
+    }
+    return foundfiles;
+}
+
+void PyDUChainTest::init()
+{
+    QString currentTest = QString(QTest::currentTestFunction());
+    if (lastTest == currentTest) {
+        kDebug() << "Already prepared assets for " << currentTest << ", skipping";
+        return;
+    } else {
+        lastTest = currentTest;
+    }
+    kDebug() << "Preparing assets for test " << currentTest;
+    
+    QDir assetModuleDir = QDir(assetsDir.absolutePath());
+    
+    if (!assetModuleDir.cd(currentTest)) {
+        kDebug() << "Asset directory " << currentTest
+                 <<  " does not exist under " << assetModuleDir.absolutePath() << ". Skipping it.";
+        return;
+    }
+    
+    kDebug() << "Searching for python files in " << assetModuleDir.absolutePath();
+    
+    QList<QString> foundfiles = FindPyFiles(assetModuleDir);
+    
+    foreach(const QString filename, foundfiles) {
+        kDebug() << "Parsing asset: " << filename;
+        DUChain::self()->updateContextForUrl(IndexedString(filename), KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
+        ICore::self()->languageController()->backgroundParser()->parseDocuments();
+    }
+    foreach(const QString filename, foundfiles) {
+        DUChain::self()->waitForUpdate(IndexedString(filename), KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
+    }
+}
+    
+
 void PyDUChainTest::initShell()
 {
     AutoTestShell::init();
@@ -78,23 +143,14 @@ void PyDUChainTest::initShell()
     ICore::self()->languageController()->backgroundParser()->parseDocuments();
     DUChain::self()->waitForUpdate(IndexedString(doc_url), KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
     
-    QFile f("/tmp/i.py");
-    f.open(QIODevice::WriteOnly);
-    f.write("def checkme(): pass\nlocalvar1 = 3\nlocalvar2 = 5\n");
-    f.write("class testclass():\n attr = 3\n");
-    f.close();
-    
-    DUChain::self()->updateContextForUrl(IndexedString("/tmp/i.py"), KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
-    ICore::self()->languageController()->backgroundParser()->parseDocuments();
-    DUChain::self()->waitForUpdate(IndexedString("/tmp/i.py"), KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
-    
     DUChain::self()->disablePersistentStorage();
     KDevelop::CodeRepresentation::setDiskChangesForbidden(true);
 }
 
 ReferencedTopDUContext PyDUChainTest::parse(const QString& code)
 {
-    TestFile* testfile = new TestFile(code + "\n", "py", 0, "/tmp/");
+    TestFile* testfile = new TestFile(code + "\n", "py", 0, testDir.absolutePath().append("/"));
+    createdFiles << testfile;
     testfile->parse((TopDUContext::Features) (TopDUContext::ForceUpdate | TopDUContext::AST) );
     testfile->waitForParsed(500);
     
@@ -106,18 +162,59 @@ ReferencedTopDUContext PyDUChainTest::parse(const QString& code)
     return 0;
 }
 
+PyDUChainTest::~PyDUChainTest()
+{
+    foreach ( TestFile* f, createdFiles ) {
+        delete f;
+    }
+    testDir.rmdir(testDir.absolutePath());
+}
+
 void PyDUChainTest::testMultiFromImport()
 {
-    ReferencedTopDUContext ctx = parse("import i.localvar1\nimport i.localvar2\na=i.localvar1\nb=i.localvar2\n");
+    QFETCH(QString, code);
+    ReferencedTopDUContext ctx = parse(code);
     QVERIFY(ctx);
     DUChainReadLocker lock;
     QList<Declaration*> a = ctx->findDeclarations(QualifiedIdentifier("a"));
     QList<Declaration*> b = ctx->findDeclarations(QualifiedIdentifier("b"));
     QVERIFY(! a.isEmpty());
     QVERIFY(! b.isEmpty());
-    kDebug() << a.first()->abstractType()->toString();
-    QVERIFY(b.first()->abstractType()->toString().endsWith("int"));
     QVERIFY(a.first()->abstractType()->toString().endsWith("int"));
+    QVERIFY(b.first()->abstractType()->toString().endsWith("int"));
+}
+
+void PyDUChainTest::testMultiFromImport_data() {
+    QTest::addColumn<QString>("code");
+    QTest::newRow("multiimport") << "import testMultiFromImport.i.localvar1\n"
+                                    "import testMultiFromImport.i.localvar2\n"
+                                    "a = testMultiFromImport.i.localvar1\n"
+                                    "b = testMultiFromImport.i.localvar2\n";
+}
+
+void PyDUChainTest::testRelativeImport()
+{
+    QFETCH(QString, code);
+    QFETCH(QString, token);
+    QFETCH(QString, type);
+    ReferencedTopDUContext ctx = parse(code);
+    QVERIFY(ctx);
+    DUChainReadLocker lock;
+    QList<Declaration*> t = ctx->findDeclarations(QualifiedIdentifier(token));
+    QVERIFY(! t.isEmpty());
+    QVERIFY(t.first()->abstractType()->toString().endsWith(type));
+}
+
+void PyDUChainTest::testRelativeImport_data() {
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<QString>("token");
+    QTest::addColumn<QString>("type");
+    QTest::newRow(".local") << "from testRelativeImport.m.sm1.go import i1" << "i1" << "int";
+    QTest::newRow(".init") << "from testRelativeImport.m.sm1.go import i2" << "i2" << "int";
+    QTest::newRow("..local") << "from testRelativeImport.m.sm1.go import i3" << "i3" << "int";
+    QTest::newRow("..init") << "from testRelativeImport.m.sm1.go import i4" << "i4" << "int";
+    QTest::newRow("..sub.local") << "from testRelativeImport.m.sm1.go import i5" << "i5" << "int";
+    QTest::newRow("..sub.init") << "from testRelativeImport.m.sm1.go import i6" << "i6" << "int";
 }
 
 void PyDUChainTest::testCrashes() {
@@ -272,7 +369,36 @@ void PyDUChainTest::testCrashes_data() {
                "(6, ''))))),"
            "(4, ''),"
            "(0, ''))))";
-    QTest::newRow("function context range crash") << "def myfunc(arg):\nfoo = 3 + \\\n[x for x in range(20)]";
+    QTest::newRow("attribute_hang") << "s = \"123\"\n"
+            "s = s.replace(u'ł', 'l').\\\n"
+            "replace(u'ó', 'o').\\\n"
+            "replace(u'ą', 'a').\\\n"
+            "replace(u'ę', 'e').\\\n"
+            "replace(u'ś', 's').\\\n"
+            "replace(u'ż', 'z').\\\n"
+            "replace(u'ź', 'z').\\\n"
+            "replace(u'ć', 'c').\\\n"
+            "replace(u'ń', 'n').\\\n"
+            "replace(u'б', 'b').\\\n"
+            "replace(u'в', 'v').\\\n"
+            "replace(u'г', 'g').\\\n"
+            "replace(u'д', 'd').\\\n"
+            "replace(u'ё', 'yo').\\\n"
+            "replace(u'ć', 'c').\\\n"
+            "replace(u'ń', 'n').\\\n"
+            "replace(u'б', 'b').\\\n"
+            "replace(u'в', 'v').\\\n"
+            "replace(u'г', 'g').\\\n"
+            "replace(u'д', 'd').\\\n"
+            "replace(u'ё', 'yo').\\\n"
+            "replace(u'ć', 'c').\\\n"
+            "replace(u'ń', 'n').\\\n"
+            "replace(u'б', 'b').\\\n"
+            "replace(u'в', 'v').\\\n"
+            "replace(u'г', 'g').\\\n"
+            "replace(u'д', 'd').\\\n"
+            "replace(u'ё', 'yo')\n";
+    QTest::newRow("function context range crash") << "def myfunc(arg):\n foo = 3 + \\\n[x for x in range(20)]";
     QTest::newRow("decorator comprehension crash") << "@implementer_only(interfaces.ISSLTransport,\n"
                  "                   *[i for i in implementedBy(tcp.Client)\n"
                  "                     if i != interfaces.ITLSTransport])\n"
@@ -343,6 +469,35 @@ void PyDUChainTest::testFlickering_data()
     QTest::newRow("declaration_flicker") << ( QStringList() << "a=2\n" << "b=3\na=2\n" ) << 1 << 2;
 }
 
+void PyDUChainTest::testCannotOverwriteBuiltins()
+{
+    QFETCH(QString, code);
+    QFETCH(QString, expectedType);
+
+    ReferencedTopDUContext ctx = parse(code);
+    DUChainWriteLocker lock;
+    QList<Declaration*> ds = ctx->findDeclarations(QualifiedIdentifier("checkme"));
+    QVERIFY(!ds.isEmpty());
+    Declaration* d = ds.first();
+    QVERIFY(d);
+    QVERIFY(d->abstractType());
+    QCOMPARE(d->abstractType()->toString(), expectedType);
+}
+
+void PyDUChainTest::testCannotOverwriteBuiltins_data()
+{
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<QString>("expectedType");
+
+    QTest::newRow("list_assign") << "class list(): pass\ncheckme = []\ncheckme.append(3)" << "list of int";
+    QTest::newRow("str_assign") << "str = 5; checkme = 'Foo'" << "str";
+    QTest::newRow("str_assign2") << "class Foo: pass\nstr = Foo; checkme = 'Foo'" << "str";
+    QTest::newRow("str_assign3") << "from testCannotOverwriteBuiltins.i import Foo as str\ncheckme = 'Foo'" << "str";
+    QTest::newRow("for") << "for str in [1, 2, 3]: pass\ncheckme = 'Foo'" << "str";
+    QTest::newRow("assert") << "assert isinstance(str, int)\ncheckme = 'Foo'" << "str";
+    QTest::newRow("assert2") << "assert isinstance(str, int)\ncheckme = 3" << "int";
+}
+
 void PyDUChainTest::testVarKWArgs()
 {
     ReferencedTopDUContext ctx = parse("def myfun(arg, *vararg, **kwarg):\n pass\n pass");
@@ -353,8 +508,8 @@ void PyDUChainTest::testVarKWArgs()
     QVERIFY(! func->findDeclarations(QualifiedIdentifier("arg")).isEmpty());
     QVERIFY(! func->findDeclarations(QualifiedIdentifier("vararg")).isEmpty());
     QVERIFY(! func->findDeclarations(QualifiedIdentifier("kwarg")).isEmpty());
-    QVERIFY(func->findDeclarations(QualifiedIdentifier("vararg")).first()->abstractType()->toString().startsWith("__kdevpythondocumentation_builtin_tuple"));
-    QVERIFY(func->findDeclarations(QualifiedIdentifier("kwarg")).first()->abstractType()->toString() == "__kdevpythondocumentation_builtin_dict");
+    QVERIFY(func->findDeclarations(QualifiedIdentifier("vararg")).first()->abstractType()->toString().startsWith("tuple"));
+    QVERIFY(func->findDeclarations(QualifiedIdentifier("kwarg")).first()->abstractType()->toString() == "dict");
 }
 
 void PyDUChainTest::testSimple()
@@ -559,18 +714,30 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("list_access_singleItem") << "some_list = []; checkme = some_list[42]" << "mixed";
     
     QTest::newRow("funccall_number") << "def foo(): return 3; \ncheckme = foo();" << "int";
-    QTest::newRow("funccall_string") << "def foo(): return 'a'; \ncheckme = foo();" << "string";
+    QTest::newRow("funccall_string") << "def foo(): return 'a'; \ncheckme = foo();" << "str";
     QTest::newRow("funccall_list") << "def foo(): return []; \ncheckme = foo();" << "list";
     QTest::newRow("funccall_dict") << "def foo(): return {}; \ncheckme = foo();" << "dict";
     
     QTest::newRow("tuple1") << "checkme, foo = 3, \"str\"" << "int";
-    QTest::newRow("tuple2") << "foo, checkme = 3, \"str\"" << "string";
+    QTest::newRow("tuple2") << "foo, checkme = 3, \"str\"" << "str";
     QTest::newRow("tuple_type") << "checkme = 1, 2" << "tuple";
     
     QTest::newRow("dict_iteritems") << "d = {1:2, 3:4}\nfor checkme, k in d.iteritems(): pass" << "int";
+
+    QTest::newRow("dict_assign_twice") << "d = dict(); d[''] = 0; d = dict(); d[''] = 0; checkme = d"
+                                       << "unsure (dict of str : int, dict)";
     
     QTest::newRow("class_method_import") << "class c:\n attr = \"foo\"\n def m():\n  return attr;\n  return 3;\ni=c()\ncheckme=i.m()" << "int";
     QTest::newRow("getsListDecorator") << "foo = [1, 2, 3]\ncheckme = foo.reverse()" << "list of int";
+
+    QTest::newRow("fromAssertIsinstance") << "class c(): pass\ncheckme = mixed()\nassert isinstance(checkme, c)\n" << "c";
+    QTest::newRow("fromAssertIsinstanceInvalid") << "class c(): pass\ncheckme = mixed()\nassert isinstance(c, checkme)\n" << "mixed";
+    QTest::newRow("fromAssertIsinstanceInvalid2") << "class c(): pass\ncheckme = mixed()\nassert isinstance(D, c)\n" << "mixed";
+    QTest::newRow("fromAssertIsinstanceInvalid3") << "checkme = int()\nassert isinstance(checkme, X)\n" << "int";
+    QTest::newRow("fromAssertIsinstanceInvalid4") << "checkme = int()\nassert isinstance(checkme)\n" << "int";
+    QTest::newRow("fromAssertType") << "class c(): pass\ncheckme = mixed()\nassert type(checkme) == c\n" << "c";
+    QTest::newRow("fromIfType") << "class c(): pass\ncheckme = mixed()\nif type(checkme) == c: pass\n" << "c";
+    QTest::newRow("fromIfIsinstance") << "class c(): pass\ncheckme = mixed()\nif isinstance(checkme, c): pass\n" << "c";
     
     QTest::newRow("diff_local_classattr") << "class c(): attr = 1\ninst=c()\ncheckme = c.attr" << "int";
     QTest::newRow("diff_local_classattr2") << "local=3\nclass c(): attr = 1\ninst=c()\ncheckme = c.local" << "mixed";
@@ -580,7 +747,7 @@ void PyDUChainTest::testTypes_data()
     
     QTest::newRow("tuple_simple") << "mytuple = 3, 5.5\ncheckme, foobar = mytuple" << "int";
     QTest::newRow("tuple_simple2") << "mytuple = 3, 5.5\nfoobar, checkme = mytuple" << "float";
-    QTest::newRow("tuple_simple3") << "mytuple = 3, 5.5, \"str\", 3, \"str\"\na, b, c, d, checkme = mytuple" << "string";
+    QTest::newRow("tuple_simple3") << "mytuple = 3, 5.5, \"str\", 3, \"str\"\na, b, c, d, checkme = mytuple" << "str";
     
     QTest::newRow("tuple_funcret") << "def myfun(): return 3, 5\ncheckme, a = myfun()" << "int";
     QTest::newRow("tuple_funcret2") << "def myfun():\n t = 3, 5\n return t\ncheckme, a = myfun()" << "int";
@@ -594,18 +761,49 @@ void PyDUChainTest::testTypes_data()
     QTest::newRow("kwarg_type") << "def myfun(**args): return args[0]\ncheckme = myfun(a=3)" << "int";
     
     QTest::newRow("tuple_listof") << "l = [(1, 2), (3, 4)]\ncheckme = l[1][0]" << "int";
+
+    QTest::newRow("getitem") << "class c:\n def __getitem__(self, slice): return 3.14\na = c()\ncheckme = a[2]" << "float";
     
     QTest::newRow("constructor_type_deduction") << "class myclass:\n"
                                                    "\tdef __init__(self, param): self.foo=param\n"
                                                    "checkme = myclass(3).foo" << "int";
-    
     QTest::newRow("functionCall_functionArg") << "def getstr(): return \"foo\"\n"
                                                  "def identity(f): return f\n"
                                                  "f1 = getstr\n"
                                                  "f2 = identity(getstr)\n"
                                                  "a = getstr()\n"
                                                  "b = f1()\n"
-                                                 "checkme = f2()\n" << "string";
+                                                 "checkme = f2()\n" << "str";
+    QTest::newRow("global_variable") << "a = 3\n"
+                                        "def f1():\n"
+                                        "  global a\n"
+                                        "  return a\n"
+                                        "checkme = f1()\n" << "int";
+    QTest::newRow("global_variable2") << "a = 3\n"
+                                        "def f1():\n"
+                                        "  global a\n"
+                                        "  a = \"str\"\n"
+                                        "  return a\n"
+                                        "checkme = f1()\n" << "str";
+    QTest::newRow("global_scope_variable") << "a = 3\n"
+                                        "def f1():\n"
+                                        "  return a\n"
+                                        "checkme = f1()\n" << "int";
+
+    QTest::newRow("top_level_vs_class_member") << "var = 3\n"
+                                                  "class myclass:\n"
+                                                  "  def __init__(self): self.var = \"str\"\n"
+                                                  "  def f1(): return var\n"
+                                                  "checkme = myclass.f1()" << "int";
+
+    // TODO PYTHON3 those got in through a failed commit... re-enable for python 3 support!
+//     QTest::newRow("vararg_before_other_args") << "def myfun(a, b, *z, x): return z[0]\n"
+//                                                  "checkme = myfun(False, False, 1, x = False)" << "int";
+//     QTest::newRow("vararg_before_other_args2") << "def myfun(a, b, *z, x): return z[3]\n"
+//                                                   "checkme = myfun(False, False, 1, 2, 3, \"str\", x = False)" << "str";
+//     QTest::newRow("vararg_constructor") << "class myclass():\n"
+//                                            "  def __init__(self, *arg): self.prop = arg[0]\n"
+//                                            "obj = myclass(3, 5); checkme = obj.prop" << "int";
 }
 
 typedef QPair<Declaration*, int> pair;
@@ -622,12 +820,7 @@ void PyDUChainTest::testImportDeclarations() {
     DUChainReadLocker lock(DUChain::lock());
     foreach ( const QString& expected, expectedDecls ) {
         bool found = false;
-        QString name;
-        QStringList split = expected.split(",");
-        name = split[0];
-//         int start, end;
-//         start = split[1].toInt();
-//         end = split[2].toInt();
+        QString name = expected;
         QList<pair> decls = ctx->allDeclarations(CursorInRevision::invalid(), ctx->topContext(), false);
         kDebug() << "FOUND DECLARATIONS:";
         foreach ( const pair& current, decls ) {
@@ -640,7 +833,7 @@ void PyDUChainTest::testImportDeclarations() {
             if ( isAliased && shouldBeAliased ) {
                 found = true; // TODO fixme
             }
-            else if ( ! shouldBeAliased && ! isAliased ) {
+            else if ( ! isAliased  && ! shouldBeAliased ) {
                 found = true;
             }
         }
@@ -653,10 +846,10 @@ void PyDUChainTest::testImportDeclarations_data() {
     QTest::addColumn<QStringList>("expectedDecls");
     QTest::addColumn<bool>("shouldBeAliased");
     
-    QTest::newRow("from_import") << "from i import checkme" << ( QStringList() << "checkme,16,23" ) << true;
-    QTest::newRow("import") << "import i" << ( QStringList() << "i,7,14" ) << false;
-    QTest::newRow("import_as") << "import i as checkme" << ( QStringList() << "checkme,14,21" ) << false;
-    QTest::newRow("from_import_as") << "from i import checkme as checkme" << ( QStringList() << "checkme,23,30" ) << true;
+    QTest::newRow("from_import") << "from testImportDeclarations.i import checkme" << ( QStringList() << "checkme" ) << true;
+    QTest::newRow("import") << "import testImportDeclarations.i" << ( QStringList() << "testImportDeclarations" ) << false;
+    QTest::newRow("import_as") << "import testImportDeclarations.i as checkme" << ( QStringList() << "checkme" ) << false;
+    QTest::newRow("from_import_as") << "from testImportDeclarations.i import checkme as checkme" << ( QStringList() << "checkme" ) << true;
 }
 
 typedef QPair<Declaration*, int> p;
@@ -753,6 +946,41 @@ void PyDUChainTest::testDecorators_data()
     QTest::newRow("two_decorators") << "@foo\n@bar(17)\ndef func(): pass" << 2 << ( QStringList() << "foo" << "bar" );
 }
 
+void PyDUChainTest::testOperators()
+{
+    QFETCH(QString, code);
+    QFETCH(QString, expectedType);
+    code.prepend("from testOperators.example import *\n\n");
+    ReferencedTopDUContext ctx = parse(code);
+    QVERIFY(ctx);
+
+    DUChainReadLocker lock(DUChain::lock());
+    TypeTestVisitor* visitor = new TypeTestVisitor();
+    visitor->ctx = TopDUContextPointer(ctx.data());
+    visitor->searchingForType = expectedType;
+    visitor->visitCode(m_ast);
+
+    QVERIFY(visitor->found);
+}
+
+void PyDUChainTest::testOperators_data()
+{
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<QString>("expectedType");
+
+    QTest::newRow("add") << "checkme = Example() + Example()" << "Add";
+    QTest::newRow("sub") << "checkme = Example() - Example()" << "Sub";
+    QTest::newRow("mul") << "checkme = Example() * Example()" << "Mul";
+    QTest::newRow("floordiv") << "checkme = Example() // Example()" << "Floordiv";
+    QTest::newRow("mod") << "checkme = Example() % Example()" << "Mod";
+    QTest::newRow("pow") << "checkme = Example() ** Example()" << "Pow";
+    QTest::newRow("lshift") << "checkme = Example() << Example()" << "Lshift";
+    QTest::newRow("rshift") << "checkme = Example() >> Example()" << "Rshift";
+    QTest::newRow("and") << "checkme = Example() & Example()" << "And";
+    QTest::newRow("xor") << "checkme = Example() ^ Example()" << "Xor";
+    QTest::newRow("or") << "checkme = Example() | Example()" << "Or";
+}
+
 void PyDUChainTest::testFunctionArgs()
 {
     ReferencedTopDUContext ctx = parse("def ASDF(arg1, arg2):\n"
@@ -791,7 +1019,7 @@ void PyDUChainTest::testInheritance()
     foreach ( const p& item, decls ) {
         kDebug() << "Checking declaration: " << item.first->identifier() << item.first->abstractType()->toString();
         if ( item.first->identifier().toString() == "checkme" ) {
-            QVERIFY(item.first->abstractType()->toString() == "__kdevpythondocumentation_builtin_int");
+            QVERIFY(item.first->abstractType()->toString() == "int");
             found = true;
         }
     }
@@ -803,7 +1031,7 @@ void PyDUChainTest::testInheritance_data()
     QTest::addColumn<QString>("code");
     
     QTest::newRow("simple") << "class A():\n\tattr = 3\n\nclass B(A):\n\tpass\n\ninst=B()\ncheckme = inst.attr";
-    QTest::newRow("context_import") << "import i\n\nclass B(i.testclass):\n\ti = 4\n\ninst=B()\ncheckme = inst.attr";
+    QTest::newRow("context_import") << "import testInheritance.i\n\nclass B(testInheritance.i.testclass):\n\ti = 4\n\ninst=B()\ncheckme = inst.attr";
 }
 
 void PyDUChainTest::testClassContextRanges()
@@ -833,11 +1061,11 @@ void PyDUChainTest::testContainerTypes()
         VariableLengthContainer* type = dynamic_cast<VariableLengthContainer*>(decls.first()->abstractType().unsafeData());
         QVERIFY(type);
         QVERIFY(type->contentType());
-        QVERIFY(type->contentType().abstractType()->toString().replace("__kdevpythondocumentation_builtin_", "") == contenttype);
+        QVERIFY(type->contentType().abstractType()->toString() == contenttype);
     }
     else {
         QVERIFY(decls.first()->abstractType());
-        QVERIFY(decls.first()->abstractType()->toString().replace("__kdevpythondocumentation_builtin_", "") == contenttype);
+        QVERIFY(decls.first()->abstractType()->toString() == contenttype);
     }
 }
 
@@ -858,9 +1086,9 @@ void PyDUChainTest::testContainerTypes_data()
     QTest::newRow("dict_of_int_call") << "checkme = dict({a:1, b:2, c:3})" << "int" << false;
     QTest::newRow("dict_generator") << "checkme = {\"Foo\":i for i in [1, 2, 3]}" << "int" << false;
     QTest::newRow("dict_access") << "list = {a:1, b:2, c:3}\ncheckme = list[0]" << "int" << true;
-    QTest::newRow("generator_attribute") << "checkme = [item.capitalize() for item in ['foobar']]" << "string" << false;
-    QTest::newRow("cannot_change_type") << "checkme = [\"Foo\", \"Bar\"]" << "string" << false;
-    QTest::newRow("cannot_change_type") << "[1, 2, 3].append(5)\ncheckme = [\"Foo\", \"Bar\"]" << "string" << false;
+    QTest::newRow("generator_attribute") << "checkme = [item.capitalize() for item in ['foobar']]" << "str" << false;
+    QTest::newRow("cannot_change_type") << "checkme = [\"Foo\", \"Bar\"]" << "str" << false;
+    QTest::newRow("cannot_change_type2") << "[1, 2, 3].append(5)\ncheckme = [\"Foo\", \"Bar\"]" << "str" << false;
     
     QTest::newRow("list_append") << "d = []\nd.append(3)\ncheckme = d[0]" << "int" << true;
 }
