@@ -20,8 +20,8 @@ func_structure = '''
             Q_ASSERT(false);
         }
 %{APPENDIX}
-        NameAst* r = dynamic_cast<NameAst*>(result);
-        if ( r ) {
+        if ( result && result->astType == Ast::NameAstType ) {
+            NameAst* r = static_cast<NameAst*>(result);
             r->startCol = r->identifier->startCol;
             r->endCol = r->identifier->endCol;
             r->startLine = r->identifier->startLine;
